<?php

namespace Controller;

use Exception;
use Traits\TResponse;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Util\Utilities;
use Service\CommentType as CommentTypeService;

class CommentType extends Simple
{

    use TResponse;

    /**
     * @var CommentTypeService
     */
    private $commentTypeService;

    /**
     * @var \Util\Utilities $utilities
     */
    private $utilities;

    public function __construct(Utilities $utilities, CommentTypeService $commentTypeService)
    {
        $this->utilities = $utilities;
        $this->commentTypeService = $commentTypeService;
    }

    public function findCommentType(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->commentTypeService->findCommentType();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveCommentType(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $id = empty($params['ID']) ? null : $params['ID'];
            $name = empty($params['NAME']) ? null : $params['NAME'];
            $control = empty($params['CONTROL']) ? null : $params['CONTROL'];
            $attachmentRequired = empty($params['ATTACHMENT_REQUIRED']) ? 'N' : $params['ATTACHMENT_REQUIRED'];
            $commentDescription = empty($params['COMMENT_DESCRIPTION']) ? null : $params['COMMENT_DESCRIPTION'];
            $this->commentTypeService->saveCommentType($id, $name, $control, $attachmentRequired, $commentDescription);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function removeCommentType(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters()['data'];
            foreach ($params as $param) {
                $commentId = $param['ID'];
                $this->commentTypeService->removeCommentType($commentId);
            }
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
