<?php

namespace Controller;

use Exception;
use Traits\TResponse;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Util\Utilities;
use Service\Priority as PriorityService;

class Priority extends Simple
{

    use TResponse;

    /**
     * @var PriorityService
     */
    private $priorityService;

    /**
     * @var \Util\Utilities $utilities
     */
    private $utilities;

    public function __construct(Utilities $utilities, PriorityService $priorityService)
    {
        $this->utilities = $utilities;
        $this->priorityService = $priorityService;
    }

    public function findIssuePriority(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->priorityService->findIssuePriority();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function removeIssuePriority(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters()['data'];
            foreach ($params as $param) {
                $priorityId = $param['ID'];
                $this->priorityService->removeIssuePriority($priorityId);
            }
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateIssuePriority(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $priorityId     = !empty($params['ID']) ? $params['ID'] : null;
            $priorityLabel  = !empty($params['LABEL']) ? $params['LABEL'] : null;
            $priorityColor  = !empty($params['COLOR']) ? $params['COLOR'] : null;
            if (empty($priorityId)) {
                $this->priorityService->createIssuePriority($priorityLabel, $priorityColor);
            } else {
                $this->priorityService->updateIssuePriority($priorityId, $priorityLabel, $priorityColor);
            }
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
