<?php


namespace Service;

use Util\InstanceProvider;
use Zeedhi\Framework\Session\Session;
use \Util\Utilities;
use Exception;
use \Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Service\Attachment as AttachmentService;


class DocumentsTemplate
{
    /**
     * @var Utilities
     */
    private $environment;
     /**
     * @var EntityManager
     */
    public $entityManager;
    private $attachmentService;



    public function __construct(AttachmentService $attachmentService)
    {
        $this->environment    = InstanceProvider::getEnvironment();
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->attachmentService = $attachmentService;

    }

    public function findDocumentsTemplate(){
        $nrOrg          = Utilities::getEnvironment()->getCurrentOrganizationId();
        $commentsRepository = InstanceProvider::getRepository()->getCommentsLocal();

        return $commentsRepository->FindDocumentsTemplate($nrOrg);
    }

    public function saveDocumentTemplate($documentTemplateId, $name, $type, $documents = array())
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $documentTemplate = null;
            if ($documentTemplateId) {
                /** @var  Entities\DocumentTemplate $documentTemplate */
                $documentTemplate = InstanceProvider::getRepository()->getDocumentTemplate()->find($documentTemplateId);
            }
            $documentTemplate = InstanceProvider::getFactory()->getDocumentTemplate()->save($documentTemplate, $name, $type);


            if (!empty($documents)) {
                $attachmentService = InstanceProvider::getService()->getAttachment();
                $attachment = $attachmentService->createDocuments($documents)[0];
                $oldAttachment = $documentTemplate->getAttachment();
                $documentTemplate->setAttachment($attachment);
                $entityManager->persist($documentTemplate);
                $entityManager->flush();
                if($oldAttachment){
                    $attachmentService->deleteAttachmentByEntities([$oldAttachment]);
                }
            }

            $entityManager = InstanceProvider::getEntityManager();
            $entityManager->persist($documentTemplate);
            $entityManager->flush();
            $connection->commit();
            return $documentTemplate->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

}