<?php

namespace Service;

use Util\InstanceProvider;
use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Repository\Task as TaskRepository;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Util\Messages;

class Task
{

    private $environment;
    private $entityManager;
    private $taskRepository;

    public function __construct(TaskRepository $taskRepository, EntityManager $entityManager)
    {
        $this->environment = InstanceProvider::getEnvironment();
        $this->entityManager = $entityManager;
        $this->taskRepository = $taskRepository;
    }

    public function findIssueClassification($filter = '')
    {
        return $this->taskRepository->findIssueClassification(
            $this->environment->getCurrentOrganizationId(),
            $filter
        );
    }

    public function findAutomaticFraming($tasks_type)
    {
        return InstanceProvider::getRepository()->getAutomaticFraming()->getAutomaticFraming($tasks_type);
    }

    public function updateTaskType(
        $id,
        $control,
        $label,
        $allowCloseIssue,
        $orderToView,
        $dayNumbers,
        $taskClassification,
        $allowPlayTask,
        $issueClassification,
        $comment,
        $targetStatusId
    ) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var Entities\TaskType $taskType */
            $taskType = InstanceProvider::getRepository()->getTaskType()->findOneBy(array('id' => $id));

            $taskType = InstanceProvider::getFactory()->getTaskType()->saveTaskTypeIssue(
                $taskType,
                $control,
                $label,
                $allowCloseIssue,
                $orderToView
            );

            $this->entityManager->persist($taskType);
            $this->entityManager->flush();

            $this->uniqueValidationsTaskType($orderToView, $taskType, $control);
            $this->saveTaskTypeClassification($taskClassification, $taskType, Entities\TaskTypeClassification::TYPE_TASK);
            $this->saveTaskTypeClassification($issueClassification, $taskType, Entities\TaskTypeClassification::TYPE_ISSUE);
            $taskType->setTimeAutoChangeStatus($dayNumbers);
            /** @var Entities\TaskStatus $targetStatus */
            $targetStatus = InstanceProvider::getRepository()->getTaskStatus()->findOneBy(array('id' => $targetStatusId));
            $taskType->setTaskStatusAutoChangeStatus($targetStatus);
            $taskType->setAllowPlayTask($allowPlayTask);
            $taskType->setDescriptionAutoChangeStatus($comment);

            $this->entityManager->persist($taskType);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $orderToView
     * @param Entities\TaskType $taskType
     * @param $control
     * @throws \Exception
     * @return bool
     */
    protected function uniqueValidationsTaskType($orderToView, $taskType, $control)
    {
        $return = false;
        $taskTypeRepository = InstanceProvider::getRepository()->getTaskType();
        $uniqueOrderToView = $taskTypeRepository->findOneBy(array(
            'organization' => $this->environment->getCurrentOrganizationId(),
            'orderToView' => $orderToView
        ));

        $uniqueControl = $taskTypeRepository->findOneBy(array(
            'organization' => $this->environment->getCurrentOrganizationId(),
            'control' => $control
        ));
        if ($uniqueOrderToView && $taskType->getId() != $uniqueOrderToView->getId()) {

            throw new \Exception(str_replace('%1', $uniqueOrderToView->getLabel(), Messages::NOT_POSSIBLE_TO_SAVE_TYPE_ORDER_TO_VIEW_ALREADY_USED));
        } else if ($control && $uniqueControl && $taskType->getId() != $uniqueControl->getId()) {

            throw new \Exception(str_replace('%1', $uniqueControl->getLabel(), Messages::NOT_POSSIBLE_TO_SAVE_TYPE_CONTROL_ALREADY_USED));
        } else {
            $return = true;
        }
        return $return;
    }

    public function saveTaskTypeClassification($classificationIds, $taskType, $type, $deleteNotIn = true)
    {
        $taskTypeClassifications = array();
        $taskTypeClassificationService = InstanceProvider::getService()->getTaskTypeClassification();
        $classificationRepository = InstanceProvider::getRepository()->getClassification();
        foreach ($classificationIds as $classificationId) {
            $classification = $classificationRepository->findOneBy(array('id' => $classificationId));
            $taskTypeClassifications[] = $taskTypeClassificationService->createTaskTypeClassification(
                $classification,
                $taskType,
                $type
            );
            $this->entityManager->flush();
        }
        if ($deleteNotIn) {
            /** @var Entities\TaskTypeClassification[] $allTaskTypeClassifications */
            $allTaskTypeClassifications = InstanceProvider::getRepository()->getTaskTypeClassification()->findBy(
                array('taskType' => $taskType, 'taskIssueClassif' => $type)
            );
            foreach ($allTaskTypeClassifications as $taskTypeClassification) {
                if (!in_array($taskTypeClassification->getClassification()->getId(), $classificationIds)) {
                    $this->entityManager->remove($taskTypeClassification);
                    $this->entityManager->flush();
                }
            }
        }
        return  $taskTypeClassifications;
    }

    public function removeTaskType($id)
    {
        InstanceProvider::getService()->getIssue()->removeTaskType($id);
    }

    public function saveTaskTypeProjectType($projectTypeIds, $taskType)
    {
        $projectTypeService = InstanceProvider::getService()->getProjectType();
        $projectTypeRepository= InstanceProvider::getRepository()->getProjectType();
        $taskTypeProjectTypes = array();
        foreach ($projectTypeIds as $projectTypeId) {
            $projectType = $projectTypeRepository->find($projectTypeId);
            $taskTypeProjectTypes[] = $projectTypeService->createTaskTypeProjectType($projectType,$taskType);
           $this->entityManager->flush();
        }
        return  $taskTypeProjectTypes;
   }

    public function findTaskStatus($filters) {
        $organizationId = $this->environment->getCurrentOrganizationId();
        $taskStatus 	= $this->taskRepository->findTaskStatus($organizationId, $filters);
        return $taskStatus;
    }

    public function saveAutomaticFraming($id, $time, $actualStatus, $destinyStatus, $taskTypeId, $description)
    {
        $connection = $this->entityManager->getConnection();
        if($actualStatus == $destinyStatus){
            throw new \Exception(Messages::ERR_SAME_STATS_BY_STATS_CONTROL);
        }
        $isNotValid = InstanceProvider::getRepository()->getAutomaticFraming()->validateFraming($taskTypeId, $actualStatus, $id);
        if($isNotValid["data"]) {
            throw new \Exception(Messages::ERR_ALREADY_HAS_FRAMING);
        }
        try {
            $connection->beginTransaction();
            if($id !== null){
                /** @var Entities\AutomaticFraming $automaticFraming */
                $entity = InstanceProvider::getRepository()->getAutomaticFraming()->find($id);
                $automaticFraming = InstanceProvider::getFactory()->getTask()->updateAutomaticFraming($entity, $id, $time, $actualStatus, $destinyStatus, $taskTypeId, $description);    
                $this->entityManager->persist($automaticFraming);
                $this->entityManager->flush();
            } else {
                $automaticFraming = InstanceProvider::getFactory()->getTask()->createAutomaticFraming($time, $actualStatus, $destinyStatus, $taskTypeId, $description);    
                $this->entityManager->persist($automaticFraming);
                $this->entityManager->flush();
            }
            $connection->commit();
            return $automaticFraming->getId();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function removeAutomaticFraming($id)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $automaticFraming = InstanceProvider::getRepository()->getAutomaticFraming()->find($id);
            $this->entityManager->remove($automaticFraming);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}
