import { TextInput, DialogService } from '@zeedhi/common';
import { IEventParam, Metadata, Http } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class StructureConfigurationController {
   public addGridRow({ component }: IEventParam<TekGrid>) {
     const newId = (component.datasource.total + 1).toString();
     component.addNewRow({ id: newId }, 'start').then(() => {
       const nameInput = Metadata.getInstance<TextInput>('LABEL_editable_undefined');
       nameInput.setFocus();
     });
   }

   public cancelEditDeleteRows({ component, event }: IEventParam<TekGrid>) {
    event?.preventDefault();
    component.cancelEditedRows();
  }

  public async editedGridRow({ component, event }: IEventParam<TekGrid>) {
    event?.preventDefault();
    const rows = component.getEditedRows();
    await Http.post('/saveStructureType', {
      DATA: rows,
    });
    component.reload();
    component.cancelEditedRows();
    component.editing = false;
  }

  private deletedAllSelectedRows({ component }: IEventParam<TekGrid>) {
    const rows = component.selectedRows;
    Http.post('/removeStructureType', {
      DATA: rows,
    });
    component.reload();
    DialogService.hide();
  }

  public async openDialogConfirmDelete({ component }: IEventParam<TekGrid>) {
    DialogService.show({
      name: 'dialog-confirm-delete',
      type: 'warning',
      title: 'Excluir Registro(s)',
      text: 'Deseja realmente excluir o(s) registro(s) selecionado(s)?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'YES',
           events: {
            click: () => this.deletedAllSelectedRows({ component }),
           },
           outline: true,
         },
         {
          name: 'no',
          component: 'ZdButton',
          label: 'NO',
           events: {
            click: () => DialogService.hide(),
           },
         },
       ],
     });
  }
}
